@@ -24,10 +24,10 @@ group.sum <- function(x, ina,ina.max = NULL,ina.min = NULL) {
 #}
 #
 #[export]
-group.mean <- function(x, ina,ina.max = max(ina)) {
-	.Defunct("group(x,ina,method=\"mean\")","Rfast")
-	#group(x,ina,ina.max=ina.max)
-}
+# group.mean <- function(x, ina,ina.max = max(ina)) {
+# 	.Defunct("group(x,ina,method=\"mean\")","Rfast")
+# 	#group(x,ina,ina.max=ina.max)
+# }
 #
 ##[export]
 #group.max <- function(x, ina,ina.min = NULL,ina.max = NULL){
